package javax.swing.text;

import java.lang.reflect.Constructor;
import sun.swing.SwingUtilities2;
import sun.reflect.misc.ReflectUtil;
import java.util.Iterator;
import java.text.AttributedCharacterIterator;
import java.util.Map;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.DecimalFormatSymbols;
import java.text.Format;
import java.text.NumberFormat;

public class NumberFormatter extends InternationalFormatter
{
    private String specialChars;
    
    public NumberFormatter() {
        this(NumberFormat.getNumberInstance());
    }
    
    public NumberFormatter(NumberFormat format) {
        super(format);
        this.setFormat(format);
        this.setAllowsInvalid(true);
        this.setCommitsOnValidEdit(false);
        this.setOverwriteMode(false);
    }
    
    @Override
    public void setFormat(Format format) {
        super.setFormat(format);
        DecimalFormatSymbols decimalFormatSymbols = this.getDecimalFormatSymbols();
        if (decimalFormatSymbols != null) {
            StringBuilder sb = new StringBuilder();
            sb.append(decimalFormatSymbols.getCurrencySymbol());
            sb.append(decimalFormatSymbols.getDecimalSeparator());
            sb.append(decimalFormatSymbols.getGroupingSeparator());
            sb.append(decimalFormatSymbols.getInfinity());
            sb.append(decimalFormatSymbols.getInternationalCurrencySymbol());
            sb.append(decimalFormatSymbols.getMinusSign());
            sb.append(decimalFormatSymbols.getMonetaryDecimalSeparator());
            sb.append(decimalFormatSymbols.getNaN());
            sb.append(decimalFormatSymbols.getPercent());
            sb.append('+');
            this.specialChars = sb.toString();
        }
        else {
            this.specialChars = "";
        }
    }
    
    @Override
    Object stringToValue(String source, Format format) throws ParseException {
        if (format == null) {
            return source;
        }
        return this.convertValueToValueClass(format.parseObject(source), this.getValueClass());
    }
    
    private Object convertValueToValueClass(Object o, Class clazz) {
        if (clazz != null && o instanceof Number) {
            Number n = (Number)o;
            if (clazz == Integer.class) {
                return Integer.valueOf(n.intValue());
            }
            if (clazz == Long.class) {
                return Long.valueOf(n.longValue());
            }
            if (clazz == Float.class) {
                return Float.valueOf(n.floatValue());
            }
            if (clazz == Double.class) {
                return Double.valueOf(n.doubleValue());
            }
            if (clazz == Byte.class) {
                return Byte.valueOf(n.byteValue());
            }
            if (clazz == Short.class) {
                return Short.valueOf(n.shortValue());
            }
        }
        return o;
    }
    
    private char getPositiveSign() {
        return '+';
    }
    
    private char getMinusSign() {
        DecimalFormatSymbols decimalFormatSymbols = this.getDecimalFormatSymbols();
        if (decimalFormatSymbols != null) {
            return decimalFormatSymbols.getMinusSign();
        }
        return '-';
    }
    
    private char getDecimalSeparator() {
        DecimalFormatSymbols decimalFormatSymbols = this.getDecimalFormatSymbols();
        if (decimalFormatSymbols != null) {
            return decimalFormatSymbols.getDecimalSeparator();
        }
        return '.';
    }
    
    private DecimalFormatSymbols getDecimalFormatSymbols() {
        Format format = this.getFormat();
        if ((format instanceof DecimalFormat)) {
            return ((DecimalFormat)format).getDecimalFormatSymbols();
        }
        return null;
    }
    
    @Override
    boolean isLegalInsertText(String s) {
        if (this.getAllowsInvalid()) {
            return true;
        }
        int i = s.length() - 1;
        while (i >= 0) {
            int char1 = s.charAt(i);
            if (!Character.isDigit((char)char1) && this.specialChars.indexOf(char1) == -1) {
                return false;
            }
            --i;
        }
        return true;
    }
    
    @Override
    boolean isLiteral(Map map) {
        if (!super.isLiteral(map)) {
            if (map == null) {
                return false;
            }
            int size = map.size();
            if (map.get(NumberFormat.Field.GROUPING_SEPARATOR) != null) {
                --size;
                if (map.get(NumberFormat.Field.INTEGER) != null) {
                    --size;
                }
            }
            if (map.get(NumberFormat.Field.EXPONENT_SYMBOL) != null) {
                --size;
            }
            if (map.get(NumberFormat.Field.PERCENT) != null) {
                --size;
            }
            if (map.get(NumberFormat.Field.PERMILLE) != null) {
                --size;
            }
            if (map.get(NumberFormat.Field.CURRENCY) != null) {
                --size;
            }
            if (map.get(NumberFormat.Field.SIGN) != null) {
                --size;
            }
            return size == 0;
        }
        else {
            return true;
        }
    }
    
    @Override
    boolean isNavigatable(int n) {
        return super.isNavigatable(n) || this.getBufferedChar(n) == this.getDecimalSeparator();
    }
    
    private NumberFormat.Field getFieldFrom(int index, int n) {
        if (this.isValidMask()) {
            int length = this.getFormattedTextField().getDocument().getLength();
            AttributedCharacterIterator iterator = this.getIterator();
            if (index >= length) {
                index += n;
            }
            while (index >= 0 && index < length) {
                iterator.setIndex(index);
                Map<AttributedCharacterIterator.Attribute, Object> attributes = iterator.getAttributes();
                if (attributes != null && attributes.size() > 0) {
                    for (Object next : attributes.keySet()) {
                        if ((next instanceof NumberFormat.Field)) {
                            return (NumberFormat.Field)next;
                        }
                        continue;
                    }
                }
                index += n;
            }
        }
        return null;
    }
    
    @Override
    void replace(DocumentFilter.FilterBypass filterBypass, int n, int n2, String s, AttributeSet set) throws BadLocationException {
        if (!this.getAllowsInvalid() && n2 == 0 && s != null && s.length() == 1 && this.toggleSignIfNecessary(filterBypass, n, s.charAt(0))) {
            return;
        }
        super.replace(filterBypass, n, n2, s, set);
        return;
    }
    
    private boolean toggleSignIfNecessary(DocumentFilter.FilterBypass filterBypass, int n, char c) throws BadLocationException {
        if (c == this.getMinusSign() || c == this.getPositiveSign()) {
            NumberFormat.Field field = this.getFieldFrom(n, -1);
            try {
                Object o;
                if (field == null || (field != NumberFormat.Field.EXPONENT && field != NumberFormat.Field.EXPONENT_SYMBOL && field != NumberFormat.Field.EXPONENT_SIGN)) {
                    o = this.toggleSign(c == this.getPositiveSign());
                }
                else {
                    o = this.toggleExponentSign(n, c);
                }
                if (o != null && this.isValidValue(o, false)) {
                    int literalCountTo = this.getLiteralCountTo(n);
                    String valueToString = this.valueToString(o);
                    filterBypass.remove(0, filterBypass.getDocument().getLength());
                    filterBypass.insertString(0, valueToString, null);
                    this.updateValue(o);
                    this.repositionCursor(this.getLiteralCountTo(n) - literalCountTo + n, 1);
                    return true;
                }
            }
            catch (ParseException ex) {
                this.invalidEdit();
            }
        }
        return false;
    }
    
    private Object toggleSign(boolean b) throws ParseException {
        Object stringToValue = super.stringToValue(this.getFormattedTextField().getText());
        if (stringToValue != null) {
            String str = stringToValue.toString();
            if (str != null && str.length() > 0) {
                if (b) {
                    if (str.charAt(0) == '-') {
                        str = str.substring(1);
                    }
                }
                else {
                    if (str.charAt(0) == '+') {
                        str = str.substring(1);
                    }
                    if (str.length() > 0 && str.charAt(0) != '-') {
                        str = "-" + str;
                    }
                }
                if (str != null) {
                    Class<Object> clazz = this.getValueClass();
                    if (clazz == null) {
                        clazz = stringToValue.getClass();
                    }
                    try {
                        ReflectUtil.checkPackageAccess(clazz);
                        SwingUtilities2.checkAccess(clazz.getModifiers());
                        Constructor<Object> constructor = clazz.getConstructor(new Class[] { String.class });
                        if (constructor != null) {
                            SwingUtilities2.checkAccess(constructor.getModifiers());
                            return constructor.newInstance(new Object[] { str });
                        }
                    }
                    catch (Throwable t) {}
                }
            }
        }
        return null;
    }
    
    private Object toggleExponentSign(int n, char c) throws BadLocationException, ParseException {
        this.getFormattedTextField().getText();
        int n2 = 0;
        int attributeStart = this.getAttributeStart(NumberFormat.Field.EXPONENT_SIGN);
        if (attributeStart >= 0) {
            n2 = 1;
            n = attributeStart;
        }
        String s;
        if (c == this.getPositiveSign()) {
            s = this.getReplaceString(n, n2, null);
        }
        else {
            s = this.getReplaceString(n, n2, new String(new char[] { c }));
        }
        return super.stringToValue(s);
    }
}
